
'Generated protocol buffer code.'
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
_sym_db = _symbol_database.Default()
from .....tensorflow.core.profiler.protobuf import diagnostics_pb2 as tensorflow_dot_core_dot_profiler_dot_protobuf_dot_diagnostics__pb2
from .....tensorflow.core.profiler.protobuf import pod_stats_pb2 as tensorflow_dot_core_dot_profiler_dot_protobuf_dot_pod__stats__pb2
DESCRIPTOR = _descriptor.FileDescriptor(name='tensorflow/core/profiler/protobuf/pod_viewer.proto', package='tensorflow.profiler', syntax='proto3', serialized_options=None, create_key=_descriptor._internal_create_key, serialized_pb=b'\n2tensorflow/core/profiler/protobuf/pod_viewer.proto\x12\x13tensorflow.profiler\x1a3tensorflow/core/profiler/protobuf/diagnostics.proto\x1a1tensorflow/core/profiler/protobuf/pod_stats.proto"#\n\x0cReplicaGroup\x12\x13\n\x0breplica_ids\x18\x01 \x03(\x03"\xac\x01\n\x0fAllReduceOpInfo\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0boccurrences\x18\x02 \x01(\r\x12\x13\n\x0bduration_us\x18\x03 \x01(\x01\x12\x11\n\tdata_size\x18\x04 \x01(\x04\x129\n\x0ereplica_groups\x18\x05 \x03(\x0b2!.tensorflow.profiler.ReplicaGroup\x12\x13\n\x0bdescription\x18\x06 \x01(\t"\xe1\x03\n\x0bPodStatsMap\x12\x10\n\x08step_num\x18\x01 \x01(\r\x12Q\n\x12pod_stats_per_core\x18\x02 \x03(\x0b25.tensorflow.profiler.PodStatsMap.PodStatsPerCoreEntry\x124\n\nchannel_db\x18\x03 \x03(\x0b2 .tensorflow.profiler.ChannelInfo\x12]\n\x19core_id_to_replica_id_map\x18\x04 \x03(\x0b2:.tensorflow.profiler.PodStatsMap.CoreIdToReplicaIdMapEntry\x12>\n\x10all_reduce_op_db\x18\x05 \x03(\x0b2$.tensorflow.profiler.AllReduceOpInfo\x1a[\n\x14PodStatsPerCoreEntry\x12\x0b\n\x03key\x18\x01 \x01(\r\x122\n\x05value\x18\x02 \x01(\x0b2#.tensorflow.profiler.PodStatsRecord:\x028\x01\x1a;\n\x19CoreIdToReplicaIdMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\r\x12\r\n\x05value\x18\x02 \x01(\r:\x028\x01"K\n\x10PodStatsSequence\x127\n\rpod_stats_map\x18\x01 \x03(\x0b2 .tensorflow.profiler.PodStatsMap"\xf0\x01\n\x0bChannelInfo\x12\x12\n\nchannel_id\x18\x01 \x01(\x03\x12\x14\n\x0csrc_core_ids\x18\x0b \x03(\r\x12\x14\n\x0cdst_core_ids\x18\x0c \x03(\r\x12\x11\n\tdata_size\x18\x04 \x01(\x04\x12\x13\n\x0bduration_us\x18\x05 \x01(\x01\x12\x13\n\x0boccurrences\x18\x06 \x01(\r\x12\x13\n\x0butilization\x18\x07 \x01(\x01\x12\x11\n\thlo_names\x18\x08 \x03(\t\x12\x15\n\rsend_delay_us\x18\t \x01(\x01\x12\x13\n\x0bdescription\x18\r \x01(\tJ\x04\x08\x02\x10\x03J\x04\x08\x03\x10\x04J\x04\x08\n\x10\x0b"$\n\x10PodViewerSummary\x12\x10\n\x08warnings\x18\x01 \x03(\t"\xb3\x01\n\x11PodViewerTopology\x12\x13\n\x0bx_dimension\x18\x01 \x01(\x05\x12\x13\n\x0by_dimension\x18\x02 \x01(\x05\x12\x13\n\x0bz_dimension\x18\x03 \x01(\x05\x12\x15\n\rhost_x_stride\x18\x04 \x01(\x05\x12\x15\n\rhost_y_stride\x18\x05 \x01(\x05\x12\x15\n\rhost_z_stride\x18\x06 \x01(\x05\x12\x1a\n\x12num_cores_per_chip\x18\x07 \x01(\x05"\xfb\x02\n\x11PodViewerDatabase\x12\x13\n\x0bdevice_type\x18\n \x01(\t\x12A\n\x12pod_stats_sequence\x18\x03 \x01(\x0b2%.tensorflow.profiler.PodStatsSequence\x126\n\x07summary\x18\x07 \x01(\x0b2%.tensorflow.profiler.PodViewerSummary\x125\n\x0bdiagnostics\x18\x08 \x01(\x0b2 .tensorflow.profiler.Diagnostics\x12G\n\x15step_breakdown_events\x18\t \x03(\x0b2(.tensorflow.profiler.StepBreakdownEvents\x128\n\x08topology\x18\x0b \x01(\x0b2&.tensorflow.profiler.PodViewerTopologyJ\x04\x08\x01\x10\x02J\x04\x08\x02\x10\x03J\x04\x08\x04\x10\x05J\x04\x08\x05\x10\x06J\x04\x08\x06\x10\x07b\x06proto3', dependencies=[tensorflow_dot_core_dot_profiler_dot_protobuf_dot_diagnostics__pb2.DESCRIPTOR, tensorflow_dot_core_dot_profiler_dot_protobuf_dot_pod__stats__pb2.DESCRIPTOR])
_REPLICAGROUP = _descriptor.Descriptor(name='ReplicaGroup', full_name='tensorflow.profiler.ReplicaGroup', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='replica_ids', full_name='tensorflow.profiler.ReplicaGroup.replica_ids', index=0, number=1, type=3, cpp_type=2, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=179, serialized_end=214)
_ALLREDUCEOPINFO = _descriptor.Descriptor(name='AllReduceOpInfo', full_name='tensorflow.profiler.AllReduceOpInfo', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='name', full_name='tensorflow.profiler.AllReduceOpInfo.name', index=0, number=1, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='occurrences', full_name='tensorflow.profiler.AllReduceOpInfo.occurrences', index=1, number=2, type=13, cpp_type=3, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='duration_us', full_name='tensorflow.profiler.AllReduceOpInfo.duration_us', index=2, number=3, type=1, cpp_type=5, label=1, has_default_value=False, default_value=float(0), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='data_size', full_name='tensorflow.profiler.AllReduceOpInfo.data_size', index=3, number=4, type=4, cpp_type=4, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='replica_groups', full_name='tensorflow.profiler.AllReduceOpInfo.replica_groups', index=4, number=5, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='description', full_name='tensorflow.profiler.AllReduceOpInfo.description', index=5, number=6, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=217, serialized_end=389)
_PODSTATSMAP_PODSTATSPERCOREENTRY = _descriptor.Descriptor(name='PodStatsPerCoreEntry', full_name='tensorflow.profiler.PodStatsMap.PodStatsPerCoreEntry', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='key', full_name='tensorflow.profiler.PodStatsMap.PodStatsPerCoreEntry.key', index=0, number=1, type=13, cpp_type=3, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='value', full_name='tensorflow.profiler.PodStatsMap.PodStatsPerCoreEntry.value', index=1, number=2, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=b'8\x01', is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=721, serialized_end=812)
_PODSTATSMAP_COREIDTOREPLICAIDMAPENTRY = _descriptor.Descriptor(name='CoreIdToReplicaIdMapEntry', full_name='tensorflow.profiler.PodStatsMap.CoreIdToReplicaIdMapEntry', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='key', full_name='tensorflow.profiler.PodStatsMap.CoreIdToReplicaIdMapEntry.key', index=0, number=1, type=13, cpp_type=3, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='value', full_name='tensorflow.profiler.PodStatsMap.CoreIdToReplicaIdMapEntry.value', index=1, number=2, type=13, cpp_type=3, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=b'8\x01', is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=814, serialized_end=873)
_PODSTATSMAP = _descriptor.Descriptor(name='PodStatsMap', full_name='tensorflow.profiler.PodStatsMap', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='step_num', full_name='tensorflow.profiler.PodStatsMap.step_num', index=0, number=1, type=13, cpp_type=3, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='pod_stats_per_core', full_name='tensorflow.profiler.PodStatsMap.pod_stats_per_core', index=1, number=2, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='channel_db', full_name='tensorflow.profiler.PodStatsMap.channel_db', index=2, number=3, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='core_id_to_replica_id_map', full_name='tensorflow.profiler.PodStatsMap.core_id_to_replica_id_map', index=3, number=4, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='all_reduce_op_db', full_name='tensorflow.profiler.PodStatsMap.all_reduce_op_db', index=4, number=5, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[_PODSTATSMAP_PODSTATSPERCOREENTRY, _PODSTATSMAP_COREIDTOREPLICAIDMAPENTRY], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=392, serialized_end=873)
_PODSTATSSEQUENCE = _descriptor.Descriptor(name='PodStatsSequence', full_name='tensorflow.profiler.PodStatsSequence', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='pod_stats_map', full_name='tensorflow.profiler.PodStatsSequence.pod_stats_map', index=0, number=1, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=875, serialized_end=950)
_CHANNELINFO = _descriptor.Descriptor(name='ChannelInfo', full_name='tensorflow.profiler.ChannelInfo', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='channel_id', full_name='tensorflow.profiler.ChannelInfo.channel_id', index=0, number=1, type=3, cpp_type=2, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='src_core_ids', full_name='tensorflow.profiler.ChannelInfo.src_core_ids', index=1, number=11, type=13, cpp_type=3, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='dst_core_ids', full_name='tensorflow.profiler.ChannelInfo.dst_core_ids', index=2, number=12, type=13, cpp_type=3, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='data_size', full_name='tensorflow.profiler.ChannelInfo.data_size', index=3, number=4, type=4, cpp_type=4, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='duration_us', full_name='tensorflow.profiler.ChannelInfo.duration_us', index=4, number=5, type=1, cpp_type=5, label=1, has_default_value=False, default_value=float(0), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='occurrences', full_name='tensorflow.profiler.ChannelInfo.occurrences', index=5, number=6, type=13, cpp_type=3, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='utilization', full_name='tensorflow.profiler.ChannelInfo.utilization', index=6, number=7, type=1, cpp_type=5, label=1, has_default_value=False, default_value=float(0), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='hlo_names', full_name='tensorflow.profiler.ChannelInfo.hlo_names', index=7, number=8, type=9, cpp_type=9, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='send_delay_us', full_name='tensorflow.profiler.ChannelInfo.send_delay_us', index=8, number=9, type=1, cpp_type=5, label=1, has_default_value=False, default_value=float(0), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='description', full_name='tensorflow.profiler.ChannelInfo.description', index=9, number=13, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=953, serialized_end=1193)
_PODVIEWERSUMMARY = _descriptor.Descriptor(name='PodViewerSummary', full_name='tensorflow.profiler.PodViewerSummary', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='warnings', full_name='tensorflow.profiler.PodViewerSummary.warnings', index=0, number=1, type=9, cpp_type=9, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=1195, serialized_end=1231)
_PODVIEWERTOPOLOGY = _descriptor.Descriptor(name='PodViewerTopology', full_name='tensorflow.profiler.PodViewerTopology', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='x_dimension', full_name='tensorflow.profiler.PodViewerTopology.x_dimension', index=0, number=1, type=5, cpp_type=1, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='y_dimension', full_name='tensorflow.profiler.PodViewerTopology.y_dimension', index=1, number=2, type=5, cpp_type=1, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='z_dimension', full_name='tensorflow.profiler.PodViewerTopology.z_dimension', index=2, number=3, type=5, cpp_type=1, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='host_x_stride', full_name='tensorflow.profiler.PodViewerTopology.host_x_stride', index=3, number=4, type=5, cpp_type=1, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='host_y_stride', full_name='tensorflow.profiler.PodViewerTopology.host_y_stride', index=4, number=5, type=5, cpp_type=1, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='host_z_stride', full_name='tensorflow.profiler.PodViewerTopology.host_z_stride', index=5, number=6, type=5, cpp_type=1, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='num_cores_per_chip', full_name='tensorflow.profiler.PodViewerTopology.num_cores_per_chip', index=6, number=7, type=5, cpp_type=1, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=1234, serialized_end=1413)
_PODVIEWERDATABASE = _descriptor.Descriptor(name='PodViewerDatabase', full_name='tensorflow.profiler.PodViewerDatabase', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='device_type', full_name='tensorflow.profiler.PodViewerDatabase.device_type', index=0, number=10, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='pod_stats_sequence', full_name='tensorflow.profiler.PodViewerDatabase.pod_stats_sequence', index=1, number=3, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='summary', full_name='tensorflow.profiler.PodViewerDatabase.summary', index=2, number=7, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='diagnostics', full_name='tensorflow.profiler.PodViewerDatabase.diagnostics', index=3, number=8, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='step_breakdown_events', full_name='tensorflow.profiler.PodViewerDatabase.step_breakdown_events', index=4, number=9, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='topology', full_name='tensorflow.profiler.PodViewerDatabase.topology', index=5, number=11, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=1416, serialized_end=1795)
_ALLREDUCEOPINFO.fields_by_name['replica_groups'].message_type = _REPLICAGROUP
_PODSTATSMAP_PODSTATSPERCOREENTRY.fields_by_name['value'].message_type = tensorflow_dot_core_dot_profiler_dot_protobuf_dot_pod__stats__pb2._PODSTATSRECORD
_PODSTATSMAP_PODSTATSPERCOREENTRY.containing_type = _PODSTATSMAP
_PODSTATSMAP_COREIDTOREPLICAIDMAPENTRY.containing_type = _PODSTATSMAP
_PODSTATSMAP.fields_by_name['pod_stats_per_core'].message_type = _PODSTATSMAP_PODSTATSPERCOREENTRY
_PODSTATSMAP.fields_by_name['channel_db'].message_type = _CHANNELINFO
_PODSTATSMAP.fields_by_name['core_id_to_replica_id_map'].message_type = _PODSTATSMAP_COREIDTOREPLICAIDMAPENTRY
_PODSTATSMAP.fields_by_name['all_reduce_op_db'].message_type = _ALLREDUCEOPINFO
_PODSTATSSEQUENCE.fields_by_name['pod_stats_map'].message_type = _PODSTATSMAP
_PODVIEWERDATABASE.fields_by_name['pod_stats_sequence'].message_type = _PODSTATSSEQUENCE
_PODVIEWERDATABASE.fields_by_name['summary'].message_type = _PODVIEWERSUMMARY
_PODVIEWERDATABASE.fields_by_name['diagnostics'].message_type = tensorflow_dot_core_dot_profiler_dot_protobuf_dot_diagnostics__pb2._DIAGNOSTICS
_PODVIEWERDATABASE.fields_by_name['step_breakdown_events'].message_type = tensorflow_dot_core_dot_profiler_dot_protobuf_dot_pod__stats__pb2._STEPBREAKDOWNEVENTS
_PODVIEWERDATABASE.fields_by_name['topology'].message_type = _PODVIEWERTOPOLOGY
DESCRIPTOR.message_types_by_name['ReplicaGroup'] = _REPLICAGROUP
DESCRIPTOR.message_types_by_name['AllReduceOpInfo'] = _ALLREDUCEOPINFO
DESCRIPTOR.message_types_by_name['PodStatsMap'] = _PODSTATSMAP
DESCRIPTOR.message_types_by_name['PodStatsSequence'] = _PODSTATSSEQUENCE
DESCRIPTOR.message_types_by_name['ChannelInfo'] = _CHANNELINFO
DESCRIPTOR.message_types_by_name['PodViewerSummary'] = _PODVIEWERSUMMARY
DESCRIPTOR.message_types_by_name['PodViewerTopology'] = _PODVIEWERTOPOLOGY
DESCRIPTOR.message_types_by_name['PodViewerDatabase'] = _PODVIEWERDATABASE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)
ReplicaGroup = _reflection.GeneratedProtocolMessageType('ReplicaGroup', (_message.Message,), {'DESCRIPTOR': _REPLICAGROUP, '__module__': 'tensorflow.core.profiler.protobuf.pod_viewer_pb2'})
_sym_db.RegisterMessage(ReplicaGroup)
AllReduceOpInfo = _reflection.GeneratedProtocolMessageType('AllReduceOpInfo', (_message.Message,), {'DESCRIPTOR': _ALLREDUCEOPINFO, '__module__': 'tensorflow.core.profiler.protobuf.pod_viewer_pb2'})
_sym_db.RegisterMessage(AllReduceOpInfo)
PodStatsMap = _reflection.GeneratedProtocolMessageType('PodStatsMap', (_message.Message,), {'PodStatsPerCoreEntry': _reflection.GeneratedProtocolMessageType('PodStatsPerCoreEntry', (_message.Message,), {'DESCRIPTOR': _PODSTATSMAP_PODSTATSPERCOREENTRY, '__module__': 'tensorflow.core.profiler.protobuf.pod_viewer_pb2'}), 'CoreIdToReplicaIdMapEntry': _reflection.GeneratedProtocolMessageType('CoreIdToReplicaIdMapEntry', (_message.Message,), {'DESCRIPTOR': _PODSTATSMAP_COREIDTOREPLICAIDMAPENTRY, '__module__': 'tensorflow.core.profiler.protobuf.pod_viewer_pb2'}), 'DESCRIPTOR': _PODSTATSMAP, '__module__': 'tensorflow.core.profiler.protobuf.pod_viewer_pb2'})
_sym_db.RegisterMessage(PodStatsMap)
_sym_db.RegisterMessage(PodStatsMap.PodStatsPerCoreEntry)
_sym_db.RegisterMessage(PodStatsMap.CoreIdToReplicaIdMapEntry)
PodStatsSequence = _reflection.GeneratedProtocolMessageType('PodStatsSequence', (_message.Message,), {'DESCRIPTOR': _PODSTATSSEQUENCE, '__module__': 'tensorflow.core.profiler.protobuf.pod_viewer_pb2'})
_sym_db.RegisterMessage(PodStatsSequence)
ChannelInfo = _reflection.GeneratedProtocolMessageType('ChannelInfo', (_message.Message,), {'DESCRIPTOR': _CHANNELINFO, '__module__': 'tensorflow.core.profiler.protobuf.pod_viewer_pb2'})
_sym_db.RegisterMessage(ChannelInfo)
PodViewerSummary = _reflection.GeneratedProtocolMessageType('PodViewerSummary', (_message.Message,), {'DESCRIPTOR': _PODVIEWERSUMMARY, '__module__': 'tensorflow.core.profiler.protobuf.pod_viewer_pb2'})
_sym_db.RegisterMessage(PodViewerSummary)
PodViewerTopology = _reflection.GeneratedProtocolMessageType('PodViewerTopology', (_message.Message,), {'DESCRIPTOR': _PODVIEWERTOPOLOGY, '__module__': 'tensorflow.core.profiler.protobuf.pod_viewer_pb2'})
_sym_db.RegisterMessage(PodViewerTopology)
PodViewerDatabase = _reflection.GeneratedProtocolMessageType('PodViewerDatabase', (_message.Message,), {'DESCRIPTOR': _PODVIEWERDATABASE, '__module__': 'tensorflow.core.profiler.protobuf.pod_viewer_pb2'})
_sym_db.RegisterMessage(PodViewerDatabase)
_PODSTATSMAP_PODSTATSPERCOREENTRY._options = None
_PODSTATSMAP_COREIDTOREPLICAIDMAPENTRY._options = None



import platform as _platform
from typing import Any, Tuple

from . import errors

def load_dynamic_library(library_filename: str) -> Tuple[errors.Status, Any]:
  import ctypes
  try:
    return errors.Status.OK(), ctypes.CDLL(str(library_filename))
  except OSError:
    return errors.NotFound(library_filename), None

def get_symbol_from_library(handle, symbol_name: str) -> Tuple[errors.Status, Any]:
  try:
    return errors.Status.OK(), getattr(handle, str(symbol_name))
  except AttributeError:
    return errors.NotFound(symbol_name), None

def format_library_file_name(name: str, version: str = None) -> str:
  name = str(name)
  version = str(version) if version is not None else ''
  if _platform.system() == "Darwin":
    if version:
      filename = "lib" + name + "." + version + ".dylib"
    else:
      filename = "lib" + name + ".dylib"
  else:
    if version:
      filename = "lib" + name + ".so." + version
    else:
      filename = "lib" + name + ".so"
  return filename





'Generated protocol buffer code.'
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
_sym_db = _symbol_database.Default()
from ....tensorflow.core.framework import function_pb2 as tensorflow_dot_core_dot_framework_dot_function__pb2
from ....tensorflow.core.framework import node_def_pb2 as tensorflow_dot_core_dot_framework_dot_node__def__pb2
from ....tensorflow.core.framework import versions_pb2 as tensorflow_dot_core_dot_framework_dot_versions__pb2
DESCRIPTOR = _descriptor.FileDescriptor(name='tensorflow/core/framework/graph.proto', package='tensorflow', syntax='proto3', serialized_options=b'\n\x18org.tensorflow.frameworkB\x0bGraphProtosP\x01ZLgithub.com/tensorflow/tensorflow/tensorflow/go/core/framework/graph_go_proto\xf8\x01\x01', create_key=_descriptor._internal_create_key, serialized_pb=b'\n%tensorflow/core/framework/graph.proto\x12\ntensorflow\x1a(tensorflow/core/framework/function.proto\x1a(tensorflow/core/framework/node_def.proto\x1a(tensorflow/core/framework/versions.proto"\x9d\x01\n\x08GraphDef\x12!\n\x04node\x18\x01 \x03(\x0b2\x13.tensorflow.NodeDef\x12(\n\x08versions\x18\x04 \x01(\x0b2\x16.tensorflow.VersionDef\x12\x13\n\x07version\x18\x03 \x01(\x05B\x02\x18\x01\x12/\n\x07library\x18\x02 \x01(\x0b2\x1e.tensorflow.FunctionDefLibraryBz\n\x18org.tensorflow.frameworkB\x0bGraphProtosP\x01ZLgithub.com/tensorflow/tensorflow/tensorflow/go/core/framework/graph_go_proto\xf8\x01\x01b\x06proto3', dependencies=[tensorflow_dot_core_dot_framework_dot_function__pb2.DESCRIPTOR, tensorflow_dot_core_dot_framework_dot_node__def__pb2.DESCRIPTOR, tensorflow_dot_core_dot_framework_dot_versions__pb2.DESCRIPTOR])
_GRAPHDEF = _descriptor.Descriptor(name='GraphDef', full_name='tensorflow.GraphDef', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='node', full_name='tensorflow.GraphDef.node', index=0, number=1, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='versions', full_name='tensorflow.GraphDef.versions', index=1, number=4, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='version', full_name='tensorflow.GraphDef.version', index=2, number=3, type=5, cpp_type=1, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=b'\x18\x01', file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='library', full_name='tensorflow.GraphDef.library', index=3, number=2, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=180, serialized_end=337)
_GRAPHDEF.fields_by_name['node'].message_type = tensorflow_dot_core_dot_framework_dot_node__def__pb2._NODEDEF
_GRAPHDEF.fields_by_name['versions'].message_type = tensorflow_dot_core_dot_framework_dot_versions__pb2._VERSIONDEF
_GRAPHDEF.fields_by_name['library'].message_type = tensorflow_dot_core_dot_framework_dot_function__pb2._FUNCTIONDEFLIBRARY
DESCRIPTOR.message_types_by_name['GraphDef'] = _GRAPHDEF
_sym_db.RegisterFileDescriptor(DESCRIPTOR)
GraphDef = _reflection.GeneratedProtocolMessageType('GraphDef', (_message.Message,), {'DESCRIPTOR': _GRAPHDEF, '__module__': 'tensorflow.core.framework.graph_pb2'})
_sym_db.RegisterMessage(GraphDef)
DESCRIPTOR._options = None
_GRAPHDEF.fields_by_name['version']._options = None




'Generated protocol buffer code.'
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
_sym_db = _symbol_database.Default()
DESCRIPTOR = _descriptor.FileDescriptor(name='tensorflow/core/profiler/protobuf/tfstreamz.proto', package='tensorflow.profiler.tfstreamz', syntax='proto3', serialized_options=None, create_key=_descriptor._internal_create_key, serialized_pb=b'\n1tensorflow/core/profiler/protobuf/tfstreamz.proto\x12\x1dtensorflow.profiler.tfstreamz"4\n\x0fPercentilePoint\x12\x12\n\npercentile\x18\x01 \x01(\x01\x12\r\n\x05value\x18\x02 \x01(\x01"\xc3\x02\n\x0bPercentiles\x12E\n\x0funit_of_measure\x18\x01 \x01(\x0e2,.tensorflow.profiler.tfstreamz.UnitOfMeasure\x12\x14\n\x0cstart_nstime\x18\x02 \x01(\x04\x12\x12\n\nend_nstime\x18\x03 \x01(\x04\x12\x11\n\tmin_value\x18\x04 \x01(\x01\x12\x11\n\tmax_value\x18\x05 \x01(\x01\x12\x0c\n\x04mean\x18\x06 \x01(\x01\x12\x0e\n\x06stddev\x18\x07 \x01(\x01\x12\x13\n\x0bnum_samples\x18\x08 \x01(\x04\x12\x15\n\rtotal_samples\x18\t \x01(\x04\x12\x13\n\x0baccumulator\x18\n \x01(\x01\x12>\n\x06points\x18\x0b \x03(\x0b2..tensorflow.profiler.tfstreamz.PercentilePoint*0\n\rUnitOfMeasure\x12\n\n\x06NUMBER\x10\x00\x12\x08\n\x04TIME\x10\x01\x12\t\n\x05BYTES\x10\x02b\x06proto3')
_UNITOFMEASURE = _descriptor.EnumDescriptor(name='UnitOfMeasure', full_name='tensorflow.profiler.tfstreamz.UnitOfMeasure', filename=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key, values=[_descriptor.EnumValueDescriptor(name='NUMBER', index=0, number=0, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='TIME', index=1, number=1, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='BYTES', index=2, number=2, serialized_options=None, type=None, create_key=_descriptor._internal_create_key)], containing_type=None, serialized_options=None, serialized_start=464, serialized_end=512)
_sym_db.RegisterEnumDescriptor(_UNITOFMEASURE)
UnitOfMeasure = enum_type_wrapper.EnumTypeWrapper(_UNITOFMEASURE)
NUMBER = 0
TIME = 1
BYTES = 2
_PERCENTILEPOINT = _descriptor.Descriptor(name='PercentilePoint', full_name='tensorflow.profiler.tfstreamz.PercentilePoint', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='percentile', full_name='tensorflow.profiler.tfstreamz.PercentilePoint.percentile', index=0, number=1, type=1, cpp_type=5, label=1, has_default_value=False, default_value=float(0), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='value', full_name='tensorflow.profiler.tfstreamz.PercentilePoint.value', index=1, number=2, type=1, cpp_type=5, label=1, has_default_value=False, default_value=float(0), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=84, serialized_end=136)
_PERCENTILES = _descriptor.Descriptor(name='Percentiles', full_name='tensorflow.profiler.tfstreamz.Percentiles', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='unit_of_measure', full_name='tensorflow.profiler.tfstreamz.Percentiles.unit_of_measure', index=0, number=1, type=14, cpp_type=8, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='start_nstime', full_name='tensorflow.profiler.tfstreamz.Percentiles.start_nstime', index=1, number=2, type=4, cpp_type=4, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='end_nstime', full_name='tensorflow.profiler.tfstreamz.Percentiles.end_nstime', index=2, number=3, type=4, cpp_type=4, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='min_value', full_name='tensorflow.profiler.tfstreamz.Percentiles.min_value', index=3, number=4, type=1, cpp_type=5, label=1, has_default_value=False, default_value=float(0), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='max_value', full_name='tensorflow.profiler.tfstreamz.Percentiles.max_value', index=4, number=5, type=1, cpp_type=5, label=1, has_default_value=False, default_value=float(0), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='mean', full_name='tensorflow.profiler.tfstreamz.Percentiles.mean', index=5, number=6, type=1, cpp_type=5, label=1, has_default_value=False, default_value=float(0), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='stddev', full_name='tensorflow.profiler.tfstreamz.Percentiles.stddev', index=6, number=7, type=1, cpp_type=5, label=1, has_default_value=False, default_value=float(0), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='num_samples', full_name='tensorflow.profiler.tfstreamz.Percentiles.num_samples', index=7, number=8, type=4, cpp_type=4, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='total_samples', full_name='tensorflow.profiler.tfstreamz.Percentiles.total_samples', index=8, number=9, type=4, cpp_type=4, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='accumulator', full_name='tensorflow.profiler.tfstreamz.Percentiles.accumulator', index=9, number=10, type=1, cpp_type=5, label=1, has_default_value=False, default_value=float(0), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='points', full_name='tensorflow.profiler.tfstreamz.Percentiles.points', index=10, number=11, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=139, serialized_end=462)
_PERCENTILES.fields_by_name['unit_of_measure'].enum_type = _UNITOFMEASURE
_PERCENTILES.fields_by_name['points'].message_type = _PERCENTILEPOINT
DESCRIPTOR.message_types_by_name['PercentilePoint'] = _PERCENTILEPOINT
DESCRIPTOR.message_types_by_name['Percentiles'] = _PERCENTILES
DESCRIPTOR.enum_types_by_name['UnitOfMeasure'] = _UNITOFMEASURE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)
PercentilePoint = _reflection.GeneratedProtocolMessageType('PercentilePoint', (_message.Message,), {'DESCRIPTOR': _PERCENTILEPOINT, '__module__': 'tensorflow.core.profiler.protobuf.tfstreamz_pb2'})
_sym_db.RegisterMessage(PercentilePoint)
Percentiles = _reflection.GeneratedProtocolMessageType('Percentiles', (_message.Message,), {'DESCRIPTOR': _PERCENTILES, '__module__': 'tensorflow.core.profiler.protobuf.tfstreamz_pb2'})
_sym_db.RegisterMessage(Percentiles)




'Generated protocol buffer code.'
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
_sym_db = _symbol_database.Default()
DESCRIPTOR = _descriptor.FileDescriptor(name='tensorflow/core/framework/versions.proto', package='tensorflow', syntax='proto3', serialized_options=b'\n\x18org.tensorflow.frameworkB\x0eVersionsProtosP\x01ZOgithub.com/tensorflow/tensorflow/tensorflow/go/core/framework/versions_go_proto\xf8\x01\x01', create_key=_descriptor._internal_create_key, serialized_pb=b'\n(tensorflow/core/framework/versions.proto\x12\ntensorflow"K\n\nVersionDef\x12\x10\n\x08producer\x18\x01 \x01(\x05\x12\x14\n\x0cmin_consumer\x18\x02 \x01(\x05\x12\x15\n\rbad_consumers\x18\x03 \x03(\x05B\x80\x01\n\x18org.tensorflow.frameworkB\x0eVersionsProtosP\x01ZOgithub.com/tensorflow/tensorflow/tensorflow/go/core/framework/versions_go_proto\xf8\x01\x01b\x06proto3')
_VERSIONDEF = _descriptor.Descriptor(name='VersionDef', full_name='tensorflow.VersionDef', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='producer', full_name='tensorflow.VersionDef.producer', index=0, number=1, type=5, cpp_type=1, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='min_consumer', full_name='tensorflow.VersionDef.min_consumer', index=1, number=2, type=5, cpp_type=1, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='bad_consumers', full_name='tensorflow.VersionDef.bad_consumers', index=2, number=3, type=5, cpp_type=1, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=56, serialized_end=131)
DESCRIPTOR.message_types_by_name['VersionDef'] = _VERSIONDEF
_sym_db.RegisterFileDescriptor(DESCRIPTOR)
VersionDef = _reflection.GeneratedProtocolMessageType('VersionDef', (_message.Message,), {'DESCRIPTOR': _VERSIONDEF, '__module__': 'tensorflow.core.framework.versions_pb2'})
_sym_db.RegisterMessage(VersionDef)
DESCRIPTOR._options = None



import struct
import re
import functools
import numpy as np
import zlib

@functools.total_ordering
class StringPiece:
  npos = -1

  def __init__(self, value=None, size=None, offset=None):
    self.set(value)
    if offset is not None:
      self.remove_prefix(offset)
    if size is not None:
      self.remove_suffix(self.size() - size)

  def data(self):
    return StringPiece(self._ptr, self._length, self._offset)

  def slice(self) -> bytearray:
    return self._ptr[self._offset:self._offset + self._length]

  def bytes(self) -> bytes:
    return bytes(self)

  def memoryview(self) -> memoryview:
    return memoryview(self.slice())

  def rfind(self, sub: bytes, start: int = None, end: int = None) -> int:
    return self.bytes().rfind(sub, start, end)

  def set(self, other):
    if other is None:
      self._ptr = bytearray()
      self._length = 0
      self._offset = 0
    elif isinstance(other, str):
      self._ptr = bytearray(other, 'utf-8')
      self._length = len(other)
      self._offset = 0
    elif isinstance(other, bytes):
      self._ptr = bytearray(other)
      self._length = len(other)
      self._offset = 0
    elif isinstance(other, bytearray):
      self._ptr = other
      self._length = len(other)
      self._offset = 0
    elif isinstance(other, StringPiece):
      self._ptr = other._ptr
      self._length = other._length
      self._offset = other._offset
    elif isinstance(other, memoryview):
      assert other.contiguous
      # assert other.readonly
      other = other.cast('b')
      self._ptr = other
      self._length = other.nbytes
      self._offset = 0
    else:
      raise TypeError("Expected stringlike")

  def at(self, pos: int, *value):
    if isinstance(pos, slice):
      start, stop, stride = pos.indices(self._length)
      assert 0 <= start <= self._length
      assert 0 <= stop <= self._length
      assert stop >= start
      assert stride == 1
      if value:
        value, = value
        assert len(value) == (stop - start)
        self._ptr[self._offset + start:self._offset + stop] = value
      else:
        return self._ptr[self._offset + start:self._offset + stop]
    else:
      assert 0 <= pos < self._length
      if value:
        value, = value
        self._ptr[self._offset + pos] = value
      else:
        return self._ptr[self._offset + pos]

  def __getitem__(self, item):
    return self.at(item)

  def __setitem__(self, pos, value):
    return self.at(pos, value)

  def length(self) -> int:
    """Returns the number of characters in the `string_view`."""
    return self._length

  def size(self) -> int:
    """Returns the number of characters in the `string_view`. Alias for `size()`."""
    return self.length()

  def empty(self) -> bool:
    """Checks if the `string_view` is empty (refers to no characters)."""
    return self._length <= 0

  def advance(self, n: int):
    if n < 0:
      if -n > self._offset:
        raise ValueError(f"Can't advance by {n}")
    elif n > 0:
      if n > self._length:
        raise ValueError(f"Can't advance by {n}")
    self._offset += n
    self._length -= n

  def remove_prefix(self, n: int):
    """Removes the first `n` characters from the `string_view`. Note that the
    underlying string is not changed, only the view."""
    if n > self._length or n < 0:
      raise ValueError(f"Can't remove {n} bytes")
    self._offset += n
    self._length -= n

  def remove_suffix(self, n: int):
    """Removes the last `n` characters from the `string_view`. Note that the
    underlying string is not changed, only the view."""
    if n > self._length or n < 0:
      raise ValueError(f"Can't remove {n} bytes")
    self._length -= n

  def find(self, target):
    return self.slice().find(string_view(target).slice())

  def rfind(self, target):
    return self.slice().rfind(string_view(target).slice())

  def find_first_of(self, chars):
    # chars = list(string_view(chars).slice())
    # me = self.slice()
    # for i, c in enumerate(me):
    #   if c in chars:
    #     return i
    # return self.npos
    pat = b'[' + re.escape(string_view(chars).slice()) + b']'
    me = self.slice()
    match = re.search(pat, me)
    if match is None:
      return self.npos
    else:
      return match.start()

  def substr(self, pos: int, n: int = npos):
    s = self.data()
    s.advance(pos)
    if n != StringPiece.npos:
      s.remove_suffix(len(s) - n)
    return s
    # s = self.slice()
    # if pos < 0:
    #   pos += len(s)
    # if n == StringPiece.npos:
    #   return s[pos:]
    # else:
    #   return s[pos:pos + n]

  def begin(self):
    return StringPiece(self._ptr, self._length, self._offset)

  def end(self):
    return StringPiece(self._ptr, 0, self._offset + self._length)

  def __bytes__(self):
    return bytes(self.slice())

  def __str__(self):
    return self.slice().decode('utf-8')

  def __repr__(self):
    return f"StringPiece({self.slice()!r})"

  def __len__(self):
    return self.length()

  def __add__(self, other):
    assert isinstance(other, int)
    r = StringPiece(self)
    r.advance(other)
    return r

  def same(self, other):
    if isinstance(other, StringPiece):
      if other._ptr is self._ptr:
        return True
      if isinstance(self._ptr, memoryview):
        if other._ptr == self._ptr:
          return True
    return False

  def __sub__(self, other):
    if isinstance(other, StringPiece):
      if not self.same(other):
        raise ValueError("Can only subtract pointers to same underlying data")
      return self._offset - other._offset
    r = StringPiece(self)
    r.advance(-other)
    return r

  def __eq__(self, other):
    # if self.same(other):
    #   if other._offset == self._offset:
    #     return other._length == self._length
    # return False
    return self.slice() == string_view(other).slice()

  def __lt__(self, other):
    # if self.same(other):
    #   return self._offset < other._offset
    # return False
    return self.slice() < string_view(other).slice()

  def compare(self, other):
    return int(self > other) - int(self < other)

  def __bool__(self):
    return not self.empty()

  def __iadd__(self, n):
    assert isinstance(n, int)
    self.advance(n)
    return self

  def __isub__(self, n):
    assert isinstance(n, int)
    self.advance(n)
    return self


def string_view(data = None, length: int = None, offset: int = None) -> StringPiece:
  return StringPiece(data, length, offset)

def get_varint_64(input: StringPiece):
  result = 0
  p = input.slice()
  i = 0
  for shift in range(0, 64, 7):
    byte = p[i]
    i += 1
    if byte & 128:
      # More bytes are present
      result |= ((byte & 127) << shift)
    else:
      result |= (byte << shift)
      input.remove_prefix(i)
      return True, result
  return False, None

def get_varint_32_ptr(data: bytes, p: int, limit: int):
  if p < limit:
    result = data[p]
    if (result & 128) == 0:
      value = result
      return p + 1, value
  return get_varint_32_ptr_fallback(data, p, limit)

def get_varint_32_ptr_fallback(data: bytes, p: int, limit: int):
  result = 0
  for shift in range(0, 29, 7):
    if p >= limit:
      break
    byte = data[p]
    p += 1
    if byte & 128:
      # More bytes are present
      result |= (byte & 127) << shift
    else:
      result |= byte << shift
      value = result
      return p, value
  return None, None

def decode_fixed_64(buffer, offset=0):
  return struct.unpack_from('<Q', buffer, offset=offset)[0]

def decode_fixed_32(buffer, offset=0):
  return struct.unpack_from('<L', buffer, offset=offset)[0]


#static const uint32 kMaskDelta = 0xa282ead8ul;
kMaskDelta = np.uint32(0xa282ead8)

class crc32c:
  @staticmethod
  def extend(init_crc: int, data, n: int) -> int:
    """Return the crc32c of concat(A, data[0,n-1]) where init_crc is the
    crc32c of some string A.  Extend() is often used to maintain the
    crc32c of a stream of data."""
    return zlib.crc32(data[:n], init_crc)

  @staticmethod
  def value(data, n: int) -> int:
    """Return the crc32c of data[0,n-1]"""
    return crc32c.extend(0, data, n)

  # // Return a masked representation of crc.
  # //
  # // Motivation: it is problematic to compute the CRC of a string that
  # // contains embedded CRCs.  Therefore we recommend that CRCs stored
  # // somewhere (e.g., in files) should be masked before being stored.
  # inline uint32 Mask(uint32 crc) {
  #   // Rotate right by 15 bits and add a constant.
  #   return ((crc >> 15) | (crc << 17)) + kMaskDelta;
  # }
  @staticmethod
  def mask(crc: int):
    """Return a masked representation of crc.

    Motivation: it is problematic to compute the CRC of a string that
    contains embedded CRCs.  Therefore we recommend that CRCs stored
    somewhere (e.g., in files) should be masked before being stored."""
    crc = np.uint32(crc)
    # Rotate right by 15 bits and add a constant.
    return np.uint32(((crc >> 15) | (crc << 17)) + kMaskDelta)

  # // Return the crc whose masked representation is masked_crc.
  # inline uint32 Unmask(uint32 masked_crc) {
  #   uint32 rot = masked_crc - kMaskDelta;
  #   return ((rot >> 17) | (rot << 15));
  # }
  @staticmethod
  def unmask(masked_crc: int):
    """Return the crc whose masked representation is masked_crc."""
    masked_crc = np.uint32(masked_crc)
    rot = masked_crc - kMaskDelta
    return np.uint32((rot >> 17) | (rot << 15))




'Generated protocol buffer code.'
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
_sym_db = _symbol_database.Default()
from ....tensorflow.lite.toco import types_pb2 as tensorflow_dot_lite_dot_toco_dot_types__pb2
DESCRIPTOR = _descriptor.FileDescriptor(name='tensorflow/lite/toco/model_flags.proto', package='toco', syntax='proto2', serialized_options=None, create_key=_descriptor._internal_create_key, serialized_pb=b'\n&tensorflow/lite/toco/model_flags.proto\x12\x04toco\x1a tensorflow/lite/toco/types.proto"5\n\x0fInputArrayShape\x12\x0c\n\x04dims\x18\x02 \x03(\x05\x12\x14\n\x0cunknown_rank\x18\x03 \x01(\x08"\x8f\x01\n\nInputArray\x12\x0c\n\x04name\x18\x01 \x01(\t\x12$\n\x05shape\x18\x06 \x01(\x0b2\x15.toco.InputArrayShape\x12\x12\n\nmean_value\x18\x03 \x01(\x02\x12\x14\n\tstd_value\x18\x04 \x01(\x02:\x011\x12#\n\tdata_type\x18\x05 \x01(\x0e2\x10.toco.IODataType"t\n\x08RnnState\x12\x13\n\x0bstate_array\x18\x01 \x01(\t\x12\x1e\n\x16back_edge_source_array\x18\x02 \x01(\t\x12\x13\n\x0bdiscardable\x18\x05 \x01(\x08\x12\x0c\n\x04size\x18\x03 \x01(\x05\x12\x10\n\x08num_dims\x18\x04 \x01(\x05"\xef\x01\n\x0fArraysExtraInfo\x12,\n\x07entries\x18\x01 \x03(\x0b2\x1b.toco.ArraysExtraInfo.Entry\x1a\xad\x01\n\x05Entry\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0bname_regexp\x18\x07 \x01(\t\x12\x0b\n\x03min\x18\x02 \x01(\x01\x12\x0b\n\x03max\x18\x03 \x01(\x01\x12#\n\tdata_type\x18\x04 \x01(\x0e2\x10.toco.IODataType\x12$\n\x05shape\x18\x05 \x01(\x0b2\x15.toco.InputArrayShape\x12\x1c\n\x14constant_float_value\x18\x06 \x01(\x02"\xc0\x04\n\nModelFlags\x12&\n\x0cinput_arrays\x18\x01 \x03(\x0b2\x10.toco.InputArray\x12\x15\n\routput_arrays\x18\x02 \x03(\t\x12\x1d\n\x15control_output_arrays\x18\x18 \x03(\t\x12\x16\n\x0evariable_batch\x18\n \x01(\x08\x12"\n\nrnn_states\x18\x0c \x03(\x0b2\x0e.toco.RnnState\x121\n\x0cmodel_checks\x18\x0e \x03(\x0b2\x1b.toco.ModelFlags.ModelCheck\x12 \n\x18allow_nonexistent_arrays\x18\x10 \x01(\x08\x12\x1d\n\x15allow_nonascii_arrays\x18\x11 \x01(\x08\x120\n\x11arrays_extra_info\x18\x12 \x01(\x0b2\x15.toco.ArraysExtraInfo\x12(\n\x1achange_concat_input_ranges\x18\x13 \x01(\x08:\x04true\x12\x17\n\x0fsaved_model_dir\x18\x14 \x01(\t\x12\x1b\n\x13saved_model_version\x18\x15 \x01(\x05\x12\x18\n\x10saved_model_tags\x18\x16 \x03(\t\x12"\n\x1asaved_model_exported_names\x18\x17 \x03(\t\x1aT\n\nModelCheck\x12\x18\n\ncount_type\x18\x01 \x01(\t:\x04None\x12\x15\n\tcount_min\x18\x02 \x01(\x05:\x02-1\x12\x15\n\tcount_max\x18\x03 \x01(\x05:\x02-1', dependencies=[tensorflow_dot_lite_dot_toco_dot_types__pb2.DESCRIPTOR])
_INPUTARRAYSHAPE = _descriptor.Descriptor(name='InputArrayShape', full_name='toco.InputArrayShape', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='dims', full_name='toco.InputArrayShape.dims', index=0, number=2, type=5, cpp_type=1, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='unknown_rank', full_name='toco.InputArrayShape.unknown_rank', index=1, number=3, type=8, cpp_type=7, label=1, has_default_value=False, default_value=False, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto2', extension_ranges=[], oneofs=[], serialized_start=82, serialized_end=135)
_INPUTARRAY = _descriptor.Descriptor(name='InputArray', full_name='toco.InputArray', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='name', full_name='toco.InputArray.name', index=0, number=1, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='shape', full_name='toco.InputArray.shape', index=1, number=6, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='mean_value', full_name='toco.InputArray.mean_value', index=2, number=3, type=2, cpp_type=6, label=1, has_default_value=False, default_value=float(0), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='std_value', full_name='toco.InputArray.std_value', index=3, number=4, type=2, cpp_type=6, label=1, has_default_value=True, default_value=float(1), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='data_type', full_name='toco.InputArray.data_type', index=4, number=5, type=14, cpp_type=8, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto2', extension_ranges=[], oneofs=[], serialized_start=138, serialized_end=281)
_RNNSTATE = _descriptor.Descriptor(name='RnnState', full_name='toco.RnnState', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='state_array', full_name='toco.RnnState.state_array', index=0, number=1, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='back_edge_source_array', full_name='toco.RnnState.back_edge_source_array', index=1, number=2, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='discardable', full_name='toco.RnnState.discardable', index=2, number=5, type=8, cpp_type=7, label=1, has_default_value=False, default_value=False, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='size', full_name='toco.RnnState.size', index=3, number=3, type=5, cpp_type=1, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='num_dims', full_name='toco.RnnState.num_dims', index=4, number=4, type=5, cpp_type=1, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto2', extension_ranges=[], oneofs=[], serialized_start=283, serialized_end=399)
_ARRAYSEXTRAINFO_ENTRY = _descriptor.Descriptor(name='Entry', full_name='toco.ArraysExtraInfo.Entry', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='name', full_name='toco.ArraysExtraInfo.Entry.name', index=0, number=1, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='name_regexp', full_name='toco.ArraysExtraInfo.Entry.name_regexp', index=1, number=7, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='min', full_name='toco.ArraysExtraInfo.Entry.min', index=2, number=2, type=1, cpp_type=5, label=1, has_default_value=False, default_value=float(0), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='max', full_name='toco.ArraysExtraInfo.Entry.max', index=3, number=3, type=1, cpp_type=5, label=1, has_default_value=False, default_value=float(0), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='data_type', full_name='toco.ArraysExtraInfo.Entry.data_type', index=4, number=4, type=14, cpp_type=8, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='shape', full_name='toco.ArraysExtraInfo.Entry.shape', index=5, number=5, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='constant_float_value', full_name='toco.ArraysExtraInfo.Entry.constant_float_value', index=6, number=6, type=2, cpp_type=6, label=1, has_default_value=False, default_value=float(0), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto2', extension_ranges=[], oneofs=[], serialized_start=468, serialized_end=641)
_ARRAYSEXTRAINFO = _descriptor.Descriptor(name='ArraysExtraInfo', full_name='toco.ArraysExtraInfo', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='entries', full_name='toco.ArraysExtraInfo.entries', index=0, number=1, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[_ARRAYSEXTRAINFO_ENTRY], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto2', extension_ranges=[], oneofs=[], serialized_start=402, serialized_end=641)
_MODELFLAGS_MODELCHECK = _descriptor.Descriptor(name='ModelCheck', full_name='toco.ModelFlags.ModelCheck', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='count_type', full_name='toco.ModelFlags.ModelCheck.count_type', index=0, number=1, type=9, cpp_type=9, label=1, has_default_value=True, default_value=b'None'.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='count_min', full_name='toco.ModelFlags.ModelCheck.count_min', index=1, number=2, type=5, cpp_type=1, label=1, has_default_value=True, default_value=(- 1), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='count_max', full_name='toco.ModelFlags.ModelCheck.count_max', index=2, number=3, type=5, cpp_type=1, label=1, has_default_value=True, default_value=(- 1), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto2', extension_ranges=[], oneofs=[], serialized_start=1136, serialized_end=1220)
_MODELFLAGS = _descriptor.Descriptor(name='ModelFlags', full_name='toco.ModelFlags', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='input_arrays', full_name='toco.ModelFlags.input_arrays', index=0, number=1, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='output_arrays', full_name='toco.ModelFlags.output_arrays', index=1, number=2, type=9, cpp_type=9, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='control_output_arrays', full_name='toco.ModelFlags.control_output_arrays', index=2, number=24, type=9, cpp_type=9, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='variable_batch', full_name='toco.ModelFlags.variable_batch', index=3, number=10, type=8, cpp_type=7, label=1, has_default_value=False, default_value=False, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='rnn_states', full_name='toco.ModelFlags.rnn_states', index=4, number=12, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='model_checks', full_name='toco.ModelFlags.model_checks', index=5, number=14, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='allow_nonexistent_arrays', full_name='toco.ModelFlags.allow_nonexistent_arrays', index=6, number=16, type=8, cpp_type=7, label=1, has_default_value=False, default_value=False, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='allow_nonascii_arrays', full_name='toco.ModelFlags.allow_nonascii_arrays', index=7, number=17, type=8, cpp_type=7, label=1, has_default_value=False, default_value=False, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='arrays_extra_info', full_name='toco.ModelFlags.arrays_extra_info', index=8, number=18, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='change_concat_input_ranges', full_name='toco.ModelFlags.change_concat_input_ranges', index=9, number=19, type=8, cpp_type=7, label=1, has_default_value=True, default_value=True, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='saved_model_dir', full_name='toco.ModelFlags.saved_model_dir', index=10, number=20, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='saved_model_version', full_name='toco.ModelFlags.saved_model_version', index=11, number=21, type=5, cpp_type=1, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='saved_model_tags', full_name='toco.ModelFlags.saved_model_tags', index=12, number=22, type=9, cpp_type=9, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='saved_model_exported_names', full_name='toco.ModelFlags.saved_model_exported_names', index=13, number=23, type=9, cpp_type=9, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[_MODELFLAGS_MODELCHECK], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto2', extension_ranges=[], oneofs=[], serialized_start=644, serialized_end=1220)
_INPUTARRAY.fields_by_name['shape'].message_type = _INPUTARRAYSHAPE
_INPUTARRAY.fields_by_name['data_type'].enum_type = tensorflow_dot_lite_dot_toco_dot_types__pb2._IODATATYPE
_ARRAYSEXTRAINFO_ENTRY.fields_by_name['data_type'].enum_type = tensorflow_dot_lite_dot_toco_dot_types__pb2._IODATATYPE
_ARRAYSEXTRAINFO_ENTRY.fields_by_name['shape'].message_type = _INPUTARRAYSHAPE
_ARRAYSEXTRAINFO_ENTRY.containing_type = _ARRAYSEXTRAINFO
_ARRAYSEXTRAINFO.fields_by_name['entries'].message_type = _ARRAYSEXTRAINFO_ENTRY
_MODELFLAGS_MODELCHECK.containing_type = _MODELFLAGS
_MODELFLAGS.fields_by_name['input_arrays'].message_type = _INPUTARRAY
_MODELFLAGS.fields_by_name['rnn_states'].message_type = _RNNSTATE
_MODELFLAGS.fields_by_name['model_checks'].message_type = _MODELFLAGS_MODELCHECK
_MODELFLAGS.fields_by_name['arrays_extra_info'].message_type = _ARRAYSEXTRAINFO
DESCRIPTOR.message_types_by_name['InputArrayShape'] = _INPUTARRAYSHAPE
DESCRIPTOR.message_types_by_name['InputArray'] = _INPUTARRAY
DESCRIPTOR.message_types_by_name['RnnState'] = _RNNSTATE
DESCRIPTOR.message_types_by_name['ArraysExtraInfo'] = _ARRAYSEXTRAINFO
DESCRIPTOR.message_types_by_name['ModelFlags'] = _MODELFLAGS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)
InputArrayShape = _reflection.GeneratedProtocolMessageType('InputArrayShape', (_message.Message,), {'DESCRIPTOR': _INPUTARRAYSHAPE, '__module__': 'tensorflow.lite.toco.model_flags_pb2'})
_sym_db.RegisterMessage(InputArrayShape)
InputArray = _reflection.GeneratedProtocolMessageType('InputArray', (_message.Message,), {'DESCRIPTOR': _INPUTARRAY, '__module__': 'tensorflow.lite.toco.model_flags_pb2'})
_sym_db.RegisterMessage(InputArray)
RnnState = _reflection.GeneratedProtocolMessageType('RnnState', (_message.Message,), {'DESCRIPTOR': _RNNSTATE, '__module__': 'tensorflow.lite.toco.model_flags_pb2'})
_sym_db.RegisterMessage(RnnState)
ArraysExtraInfo = _reflection.GeneratedProtocolMessageType('ArraysExtraInfo', (_message.Message,), {'Entry': _reflection.GeneratedProtocolMessageType('Entry', (_message.Message,), {'DESCRIPTOR': _ARRAYSEXTRAINFO_ENTRY, '__module__': 'tensorflow.lite.toco.model_flags_pb2'}), 'DESCRIPTOR': _ARRAYSEXTRAINFO, '__module__': 'tensorflow.lite.toco.model_flags_pb2'})
_sym_db.RegisterMessage(ArraysExtraInfo)
_sym_db.RegisterMessage(ArraysExtraInfo.Entry)
ModelFlags = _reflection.GeneratedProtocolMessageType('ModelFlags', (_message.Message,), {'ModelCheck': _reflection.GeneratedProtocolMessageType('ModelCheck', (_message.Message,), {'DESCRIPTOR': _MODELFLAGS_MODELCHECK, '__module__': 'tensorflow.lite.toco.model_flags_pb2'}), 'DESCRIPTOR': _MODELFLAGS, '__module__': 'tensorflow.lite.toco.model_flags_pb2'})
_sym_db.RegisterMessage(ModelFlags)
_sym_db.RegisterMessage(ModelFlags.ModelCheck)




'Generated protocol buffer code.'
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
_sym_db = _symbol_database.Default()
DESCRIPTOR = _descriptor.FileDescriptor(name='tensorflow/core/framework/tensor_shape.proto', package='tensorflow', syntax='proto3', serialized_options=b'\n\x18org.tensorflow.frameworkB\x11TensorShapeProtosP\x01ZSgithub.com/tensorflow/tensorflow/tensorflow/go/core/framework/tensor_shape_go_proto\xf8\x01\x01', create_key=_descriptor._internal_create_key, serialized_pb=b'\n,tensorflow/core/framework/tensor_shape.proto\x12\ntensorflow"z\n\x10TensorShapeProto\x12-\n\x03dim\x18\x02 \x03(\x0b2 .tensorflow.TensorShapeProto.Dim\x12\x14\n\x0cunknown_rank\x18\x03 \x01(\x08\x1a!\n\x03Dim\x12\x0c\n\x04size\x18\x01 \x01(\x03\x12\x0c\n\x04name\x18\x02 \x01(\tB\x87\x01\n\x18org.tensorflow.frameworkB\x11TensorShapeProtosP\x01ZSgithub.com/tensorflow/tensorflow/tensorflow/go/core/framework/tensor_shape_go_proto\xf8\x01\x01b\x06proto3')
_TENSORSHAPEPROTO_DIM = _descriptor.Descriptor(name='Dim', full_name='tensorflow.TensorShapeProto.Dim', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='size', full_name='tensorflow.TensorShapeProto.Dim.size', index=0, number=1, type=3, cpp_type=2, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='name', full_name='tensorflow.TensorShapeProto.Dim.name', index=1, number=2, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=149, serialized_end=182)
_TENSORSHAPEPROTO = _descriptor.Descriptor(name='TensorShapeProto', full_name='tensorflow.TensorShapeProto', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='dim', full_name='tensorflow.TensorShapeProto.dim', index=0, number=2, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='unknown_rank', full_name='tensorflow.TensorShapeProto.unknown_rank', index=1, number=3, type=8, cpp_type=7, label=1, has_default_value=False, default_value=False, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[_TENSORSHAPEPROTO_DIM], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=60, serialized_end=182)
_TENSORSHAPEPROTO_DIM.containing_type = _TENSORSHAPEPROTO
_TENSORSHAPEPROTO.fields_by_name['dim'].message_type = _TENSORSHAPEPROTO_DIM
DESCRIPTOR.message_types_by_name['TensorShapeProto'] = _TENSORSHAPEPROTO
_sym_db.RegisterFileDescriptor(DESCRIPTOR)
TensorShapeProto = _reflection.GeneratedProtocolMessageType('TensorShapeProto', (_message.Message,), {'Dim': _reflection.GeneratedProtocolMessageType('Dim', (_message.Message,), {'DESCRIPTOR': _TENSORSHAPEPROTO_DIM, '__module__': 'tensorflow.core.framework.tensor_shape_pb2'}), 'DESCRIPTOR': _TENSORSHAPEPROTO, '__module__': 'tensorflow.core.framework.tensor_shape_pb2'})
_sym_db.RegisterMessage(TensorShapeProto)
_sym_db.RegisterMessage(TensorShapeProto.Dim)
DESCRIPTOR._options = None



from . import table_options
import numpy as np

from typing import List

#import tensorflow_checkpoint_reader.core as core
from . import core
from . import coding
from . import strings

class BlockBuilder:
  #  // Reset the contents as if the BlockBuilder was just constructed.
  #  void Reset();
  #
  #  // REQUIRES: Finish() has not been called since the last call to Reset().
  #  // REQUIRES: key is larger than any previously added key
  #  void Add(const StringPiece& key, const StringPiece& value);
  #
  #  // Finish building the block and return a slice that refers to the
  #  // block contents.  The returned slice will remain valid for the
  #  // lifetime of this builder or until Reset() is called.
  #  StringPiece Finish();
  #
  #  // Returns an estimate of the current (uncompressed) size of the block
  #  // we are building.
  #  size_t CurrentSizeEstimate() const;
  #
  #  // Return true iff no entries have been added since the last Reset()
  #  bool empty() const { return buffer_.empty(); }
  #
  # private:
  #  const Options* options_;
  #  string buffer_;                 // Destination buffer
  #  std::vector<uint32> restarts_;  // Restart points
  #  int counter_;                   // Number of entries emitted since restart
  #  bool finished_;                 // Has Finish() been called?
  #  string last_key_;
  #
  #  // No copying allowed
  #  BlockBuilder(const BlockBuilder&);
  #  void operator=(const BlockBuilder&);


  # namespace tensorflow {
  # namespace table {
  #
  # BlockBuilder::BlockBuilder(const Options* options)
  #     : options_(options), restarts_(), counter_(0), finished_(false) {
  #   assert(options->block_restart_interval >= 1);
  #   restarts_.push_back(0);  // First restart point is at offset 0
  # }

  restarts_: List[int]

  def __init__(self, options: table_options.Options):
    self.options_ = options
    self.buffer_ = bytearray() # Destination buffer
    self.restarts_ = []
    self.counter_ = 0
    self.finished_ = False
    self.last_key_ = bytearray()
    assert options.block_restart_interval >= 1
    self.restarts_.append(0) # First restart point is at offset 0

  # void BlockBuilder::Reset() {
  #   buffer_.clear();
  #   restarts_.clear();
  #   restarts_.push_back(0);  // First restart point is at offset 0
  #   counter_ = 0;
  #   finished_ = false;
  #   last_key_.clear();
  # }
  def reset(self):
    """Reset the contents as if the BlockBuilder was just constructed."""
    self.buffer_.clear()
    self.restarts_.clear()
    self.restarts_.append(0) # First restart point is at offset 0
    self.counter_ = 0
    self.finished_ = False
    self.last_key_.clear()

  # size_t BlockBuilder::CurrentSizeEstimate() const {
  #   return (buffer_.size() +                     // Raw data buffer
  #           restarts_.size() * sizeof(uint32) +  // Restart array
  #           sizeof(uint32));                     // Restart array length
  # }
  def current_size_estimate(self):
    return (len(self.buffer_) +        # Raw data buffer
            len(self.restarts_) * 4 +  # Restart array
            4                          # Restart array length
            )


  def empty(self):
    """Return true iff no entries have been added since the last Reset()"""
    return len(self.buffer_) <= 0

  # StringPiece BlockBuilder::Finish() {
  #   // Append restart array
  #   CHECK_LE(restarts_.size(), std::numeric_limits<uint32_t>::max());
  #   for (const auto r : restarts_) {
  #     core::PutFixed32(&buffer_, r);
  #   }
  #   // Downcast safe because of the CHECK.
  #   core::PutFixed32(&buffer_, static_cast<uint32_t>(restarts_.size()));
  #   finished_ = true;
  #   return StringPiece(buffer_);
  # }
  def finish(self):
    """Finish building the block and return a slice that refers to the
    block contents.  The returned slice will remain valid for the
    lifetime of this builder or until Reset() is called.
    """
    # Append restart array
    # CHECK_LE(restarts_.size(), std::numeric_limits<uint32_t>::max());
    assert len(self.restarts_) <= np.iinfo(np.uint32).max
    # for (const auto r : restarts_) {
    #   core::PutFixed32(&buffer_, r);
    # }
    for r in self.restarts_:
      coding.put_fixed32(self.buffer_, r)
    # // Downcast safe because of the CHECK.
    # core::PutFixed32(&buffer_, static_cast<uint32_t>(restarts_.size()));
    coding.put_fixed32(self.buffer_, np.uint32(len(self.restarts_)))
    # finished_ = true;
    self.finished_ = True
    # return StringPiece(buffer_);
    return core.StringPiece(self.buffer_)

  # void BlockBuilder::Add(const StringPiece& key, const StringPiece& value) {
  #   StringPiece last_key_piece(last_key_);
  #   assert(!finished_);
  #   assert(counter_ <= options_->block_restart_interval);
  #   assert(buffer_.empty()  // No values yet?
  #          || key.compare(last_key_piece) > 0);
  #   size_t shared = 0;
  #   if (counter_ < options_->block_restart_interval) {
  #     // See how much sharing to do with previous string
  #     const size_t min_length = std::min(last_key_piece.size(), key.size());
  #     while ((shared < min_length) && (last_key_piece[shared] == key[shared])) {
  #       shared++;
  #     }
  #   } else {
  #     // Restart compression
  #     CHECK_LE(buffer_.size(), std::numeric_limits<uint32_t>::max());
  #     restarts_.push_back(static_cast<uint32_t>(buffer_.size()));
  #     counter_ = 0;
  #   }
  #   const size_t non_shared = key.size() - shared;
  #
  #   CHECK_LE(shared, std::numeric_limits<uint32_t>::max());
  #   CHECK_LE(non_shared, std::numeric_limits<uint32_t>::max());
  #   CHECK_LE(value.size(), std::numeric_limits<uint32_t>::max());
  #
  #   // Add "<shared><non_shared><value_size>" to buffer_
  #   core::PutVarint32(&buffer_, static_cast<uint32_t>(shared));
  #   core::PutVarint32(&buffer_, static_cast<uint32_t>(non_shared));
  #   core::PutVarint32(&buffer_, static_cast<uint32_t>(value.size()));
  #
  #   // Add string delta to buffer_ followed by value
  #   buffer_.append(key.data() + shared, non_shared);
  #   buffer_.append(value.data(), static_cast<uint32_t>(value.size()));
  #
  #   // Update state
  #   last_key_.resize(shared);
  #   last_key_.append(key.data() + shared, non_shared);
  #   assert(StringPiece(last_key_) == key);
  #   counter_++;
  # }
  def add(self, key, value):
    """
    REQUIRES: Finish() has not been called since the last call to Reset().
    REQUIRES: key is larger than any previously added key
    """
    key = core.string_view(key)
    value = core.string_view(value)
    # StringPiece last_key_piece(last_key_);
    last_key_piece = core.StringPiece(self.last_key_)
    # assert(!finished_);
    assert not self.finished_
    # assert(counter_ <= options_->block_restart_interval);
    assert self.counter_ <= self.options_.block_restart_interval
    # assert(buffer_.empty()  // No values yet?
    #        || key.compare(last_key_piece) > 0);
    assert (len(self.buffer_) <= 0  # No values yet?
            or key.compare(last_key_piece) > 0)

    # size_t shared = 0;
    shared = 0
    # if (counter_ < options_->block_restart_interval) {
    if self.counter_ < self.options_.block_restart_interval:
      # // See how much sharing to do with previous string
      # const size_t min_length = std::min(last_key_piece.size(), key.size());
      min_length = min(last_key_piece.size(), key.size())
      # while ((shared < min_length) && (last_key_piece[shared] == key[shared])) {
      #   shared++;
      # }
      while (shared < min_length) and (last_key_piece[shared] == key[shared]):
        shared += 1
    # } else {
    else:
      # // Restart compression
      # CHECK_LE(buffer_.size(), std::numeric_limits<uint32_t>::max());
      assert len(self.buffer_) <= np.iinfo(np.uint32).max
      # restarts_.push_back(static_cast<uint32_t>(buffer_.size()));
      self.restarts_.append(np.uint32(len(self.buffer_)))
      # counter_ = 0;
      self.counter_ = 0
    # }
    # const size_t non_shared = key.size() - shared;
    non_shared = key.size() - shared

    # CHECK_LE(shared, std::numeric_limits<uint32_t>::max());
    # CHECK_LE(non_shared, std::numeric_limits<uint32_t>::max());
    # CHECK_LE(value.size(), std::numeric_limits<uint32_t>::max());
    assert shared <= np.iinfo(np.uint32).max
    assert non_shared <= np.iinfo(np.uint32).max
    assert value.size() <= np.iinfo(np.uint32).max

    # // Add "<shared><non_shared><value_size>" to buffer_
    # core::PutVarint32(&buffer_, static_cast<uint32_t>(shared));
    # core::PutVarint32(&buffer_, static_cast<uint32_t>(non_shared));
    # core::PutVarint32(&buffer_, static_cast<uint32_t>(value.size()));
    coding.put_varint32(self.buffer_, shared)
    coding.put_varint32(self.buffer_, non_shared)
    coding.put_varint32(self.buffer_, value.size())

    # // Add string delta to buffer_ followed by value
    # buffer_.append(key.data() + shared, non_shared);
    # buffer_.append(value.data(), static_cast<uint32_t>(value.size()));
    self.buffer_.extend(key[shared:][:non_shared])
    self.buffer_.extend(value.bytes())

    # // Update state
    # last_key_.resize(shared);
    # last_key_.append(key.data() + shared, non_shared);
    # assert(StringPiece(last_key_) == key);
    # counter_++;
    strings.resize(self.last_key_, shared)
    self.last_key_.extend(key[shared:][:non_shared])
    assert core.StringPiece(self.last_key_) == key
    self.counter_ += 1

  # }  // namespace table
  # }  // namespace tensorflow




'Generated protocol buffer code.'
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
_sym_db = _symbol_database.Default()
DESCRIPTOR = _descriptor.FileDescriptor(name='tensorflow/core/profiler/protobuf/op_metrics.proto', package='tensorflow.profiler', syntax='proto3', serialized_options=None, create_key=_descriptor._internal_create_key, serialized_pb=b'\n2tensorflow/core/profiler/protobuf/op_metrics.proto\x12\x13tensorflow.profiler"\xc3\x01\n\x0eLayoutAnalysis\x12A\n\ndimensions\x18\x01 \x03(\x0b2-.tensorflow.profiler.LayoutAnalysis.Dimension\x1an\n\tDimension\x12\x0c\n\x04size\x18\x01 \x01(\x05\x12\x11\n\talignment\x18\x02 \x01(\x05\x12@\n\tsemantics\x18\x03 \x01(\x0e2-.tensorflow.profiler.LayoutDimensionSemantics"\xba\x05\n\tOpMetrics\x12\x15\n\rhlo_module_id\x18\r \x01(\x04\x12\x0c\n\x04name\x18\x06 \x01(\t\x12\x11\n\tlong_name\x18\x14 \x01(\t\x12\x10\n\x08category\x18\x0b \x01(\t\x12\x12\n\nprovenance\x18\x0c \x01(\t\x12\x10\n\x08is_eager\x18\x12 \x01(\x08\x12\x13\n\x0boccurrences\x18\x03 \x01(\r\x12\x0f\n\x07time_ps\x18\x07 \x01(\x04\x12\x13\n\x0bmin_time_ps\x18\x11 \x01(\x04\x12\x14\n\x0cself_time_ps\x18\x01 \x01(\x04\x12\r\n\x05flops\x18\x02 \x01(\x04\x12\x16\n\x0ebytes_accessed\x18\x05 \x01(\x04\x12P\n\x19memory_accessed_breakdown\x18\x13 \x03(\x0b2-.tensorflow.profiler.OpMetrics.MemoryAccessed\x12\x14\n\x0cdma_stall_ps\x18\n \x01(\x04\x123\n\x06layout\x18\x0e \x01(\x0b2#.tensorflow.profiler.LayoutAnalysis\x12\x19\n\x11deduplicated_name\x18\x0f \x01(\t\x122\n\x08children\x18\x10 \x01(\x0b2 .tensorflow.profiler.OpMetricsDb\x1a\xc6\x01\n\x0eMemoryAccessed\x12S\n\x0eoperation_type\x18\x01 \x01(\x0e2;.tensorflow.profiler.OpMetrics.MemoryAccessed.OperationType\x12\x14\n\x0cmemory_space\x18\x02 \x01(\x04\x12\x16\n\x0ebytes_accessed\x18\x03 \x01(\x04"1\n\rOperationType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x08\n\x04READ\x10\x01\x12\t\n\x05WRITE\x10\x02J\x04\x08\x04\x10\x05J\x04\x08\x08\x10\tJ\x04\x08\t\x10\n"D\n\x0ePrecisionStats\x12\x18\n\x10compute_16bit_ps\x18\x01 \x01(\x04\x12\x18\n\x10compute_32bit_ps\x18\x02 \x01(\x04"\xbc\x02\n\x0bOpMetricsDb\x122\n\nmetrics_db\x18\n \x03(\x0b2\x1e.tensorflow.profiler.OpMetrics\x12)\n!total_host_infeed_enq_duration_ps\x18\x02 \x01(\x04\x125\n-total_host_infeed_enq_start_timestamp_ps_diff\x18\x03 \x01(\x04\x12\x15\n\rtotal_time_ps\x18\x0b \x01(\x04\x12\x18\n\x10total_op_time_ps\x18\x0c \x01(\x04\x12<\n\x0fprecision_stats\x18\r \x01(\x0b2#.tensorflow.profiler.PrecisionStatsJ\x04\x08\x01\x10\x02J\x04\x08\x04\x10\x05J\x04\x08\x05\x10\x06J\x04\x08\x06\x10\x07J\x04\x08\x07\x10\x08J\x04\x08\x08\x10\tJ\x04\x08\t\x10\n*V\n\x18LayoutDimensionSemantics\x12\x15\n\x11UNKNOWN_SEMANTICS\x10\x00\x12\x0b\n\x07FEATURE\x10\x01\x12\t\n\x05BATCH\x10\x02\x12\x0b\n\x07SPATIAL\x10\x03b\x06proto3')
_LAYOUTDIMENSIONSEMANTICS = _descriptor.EnumDescriptor(name='LayoutDimensionSemantics', full_name='tensorflow.profiler.LayoutDimensionSemantics', filename=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key, values=[_descriptor.EnumValueDescriptor(name='UNKNOWN_SEMANTICS', index=0, number=0, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='FEATURE', index=1, number=1, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='BATCH', index=2, number=2, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='SPATIAL', index=3, number=3, serialized_options=None, type=None, create_key=_descriptor._internal_create_key)], containing_type=None, serialized_options=None, serialized_start=1363, serialized_end=1449)
_sym_db.RegisterEnumDescriptor(_LAYOUTDIMENSIONSEMANTICS)
LayoutDimensionSemantics = enum_type_wrapper.EnumTypeWrapper(_LAYOUTDIMENSIONSEMANTICS)
UNKNOWN_SEMANTICS = 0
FEATURE = 1
BATCH = 2
SPATIAL = 3
_OPMETRICS_MEMORYACCESSED_OPERATIONTYPE = _descriptor.EnumDescriptor(name='OperationType', full_name='tensorflow.profiler.OpMetrics.MemoryAccessed.OperationType', filename=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key, values=[_descriptor.EnumValueDescriptor(name='UNKNOWN', index=0, number=0, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='READ', index=1, number=1, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='WRITE', index=2, number=2, serialized_options=None, type=None, create_key=_descriptor._internal_create_key)], containing_type=None, serialized_options=None, serialized_start=905, serialized_end=954)
_sym_db.RegisterEnumDescriptor(_OPMETRICS_MEMORYACCESSED_OPERATIONTYPE)
_LAYOUTANALYSIS_DIMENSION = _descriptor.Descriptor(name='Dimension', full_name='tensorflow.profiler.LayoutAnalysis.Dimension', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='size', full_name='tensorflow.profiler.LayoutAnalysis.Dimension.size', index=0, number=1, type=5, cpp_type=1, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='alignment', full_name='tensorflow.profiler.LayoutAnalysis.Dimension.alignment', index=1, number=2, type=5, cpp_type=1, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='semantics', full_name='tensorflow.profiler.LayoutAnalysis.Dimension.semantics', index=2, number=3, type=14, cpp_type=8, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=161, serialized_end=271)
_LAYOUTANALYSIS = _descriptor.Descriptor(name='LayoutAnalysis', full_name='tensorflow.profiler.LayoutAnalysis', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='dimensions', full_name='tensorflow.profiler.LayoutAnalysis.dimensions', index=0, number=1, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[_LAYOUTANALYSIS_DIMENSION], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=76, serialized_end=271)
_OPMETRICS_MEMORYACCESSED = _descriptor.Descriptor(name='MemoryAccessed', full_name='tensorflow.profiler.OpMetrics.MemoryAccessed', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='operation_type', full_name='tensorflow.profiler.OpMetrics.MemoryAccessed.operation_type', index=0, number=1, type=14, cpp_type=8, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='memory_space', full_name='tensorflow.profiler.OpMetrics.MemoryAccessed.memory_space', index=1, number=2, type=4, cpp_type=4, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='bytes_accessed', full_name='tensorflow.profiler.OpMetrics.MemoryAccessed.bytes_accessed', index=2, number=3, type=4, cpp_type=4, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[_OPMETRICS_MEMORYACCESSED_OPERATIONTYPE], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=756, serialized_end=954)
_OPMETRICS = _descriptor.Descriptor(name='OpMetrics', full_name='tensorflow.profiler.OpMetrics', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='hlo_module_id', full_name='tensorflow.profiler.OpMetrics.hlo_module_id', index=0, number=13, type=4, cpp_type=4, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='name', full_name='tensorflow.profiler.OpMetrics.name', index=1, number=6, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='long_name', full_name='tensorflow.profiler.OpMetrics.long_name', index=2, number=20, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='category', full_name='tensorflow.profiler.OpMetrics.category', index=3, number=11, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='provenance', full_name='tensorflow.profiler.OpMetrics.provenance', index=4, number=12, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='is_eager', full_name='tensorflow.profiler.OpMetrics.is_eager', index=5, number=18, type=8, cpp_type=7, label=1, has_default_value=False, default_value=False, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='occurrences', full_name='tensorflow.profiler.OpMetrics.occurrences', index=6, number=3, type=13, cpp_type=3, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='time_ps', full_name='tensorflow.profiler.OpMetrics.time_ps', index=7, number=7, type=4, cpp_type=4, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='min_time_ps', full_name='tensorflow.profiler.OpMetrics.min_time_ps', index=8, number=17, type=4, cpp_type=4, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='self_time_ps', full_name='tensorflow.profiler.OpMetrics.self_time_ps', index=9, number=1, type=4, cpp_type=4, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='flops', full_name='tensorflow.profiler.OpMetrics.flops', index=10, number=2, type=4, cpp_type=4, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='bytes_accessed', full_name='tensorflow.profiler.OpMetrics.bytes_accessed', index=11, number=5, type=4, cpp_type=4, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='memory_accessed_breakdown', full_name='tensorflow.profiler.OpMetrics.memory_accessed_breakdown', index=12, number=19, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='dma_stall_ps', full_name='tensorflow.profiler.OpMetrics.dma_stall_ps', index=13, number=10, type=4, cpp_type=4, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='layout', full_name='tensorflow.profiler.OpMetrics.layout', index=14, number=14, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='deduplicated_name', full_name='tensorflow.profiler.OpMetrics.deduplicated_name', index=15, number=15, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='children', full_name='tensorflow.profiler.OpMetrics.children', index=16, number=16, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[_OPMETRICS_MEMORYACCESSED], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=274, serialized_end=972)
_PRECISIONSTATS = _descriptor.Descriptor(name='PrecisionStats', full_name='tensorflow.profiler.PrecisionStats', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='compute_16bit_ps', full_name='tensorflow.profiler.PrecisionStats.compute_16bit_ps', index=0, number=1, type=4, cpp_type=4, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='compute_32bit_ps', full_name='tensorflow.profiler.PrecisionStats.compute_32bit_ps', index=1, number=2, type=4, cpp_type=4, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=974, serialized_end=1042)
_OPMETRICSDB = _descriptor.Descriptor(name='OpMetricsDb', full_name='tensorflow.profiler.OpMetricsDb', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='metrics_db', full_name='tensorflow.profiler.OpMetricsDb.metrics_db', index=0, number=10, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='total_host_infeed_enq_duration_ps', full_name='tensorflow.profiler.OpMetricsDb.total_host_infeed_enq_duration_ps', index=1, number=2, type=4, cpp_type=4, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='total_host_infeed_enq_start_timestamp_ps_diff', full_name='tensorflow.profiler.OpMetricsDb.total_host_infeed_enq_start_timestamp_ps_diff', index=2, number=3, type=4, cpp_type=4, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='total_time_ps', full_name='tensorflow.profiler.OpMetricsDb.total_time_ps', index=3, number=11, type=4, cpp_type=4, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='total_op_time_ps', full_name='tensorflow.profiler.OpMetricsDb.total_op_time_ps', index=4, number=12, type=4, cpp_type=4, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='precision_stats', full_name='tensorflow.profiler.OpMetricsDb.precision_stats', index=5, number=13, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=1045, serialized_end=1361)
_LAYOUTANALYSIS_DIMENSION.fields_by_name['semantics'].enum_type = _LAYOUTDIMENSIONSEMANTICS
_LAYOUTANALYSIS_DIMENSION.containing_type = _LAYOUTANALYSIS
_LAYOUTANALYSIS.fields_by_name['dimensions'].message_type = _LAYOUTANALYSIS_DIMENSION
_OPMETRICS_MEMORYACCESSED.fields_by_name['operation_type'].enum_type = _OPMETRICS_MEMORYACCESSED_OPERATIONTYPE
_OPMETRICS_MEMORYACCESSED.containing_type = _OPMETRICS
_OPMETRICS_MEMORYACCESSED_OPERATIONTYPE.containing_type = _OPMETRICS_MEMORYACCESSED
_OPMETRICS.fields_by_name['memory_accessed_breakdown'].message_type = _OPMETRICS_MEMORYACCESSED
_OPMETRICS.fields_by_name['layout'].message_type = _LAYOUTANALYSIS
_OPMETRICS.fields_by_name['children'].message_type = _OPMETRICSDB
_OPMETRICSDB.fields_by_name['metrics_db'].message_type = _OPMETRICS
_OPMETRICSDB.fields_by_name['precision_stats'].message_type = _PRECISIONSTATS
DESCRIPTOR.message_types_by_name['LayoutAnalysis'] = _LAYOUTANALYSIS
DESCRIPTOR.message_types_by_name['OpMetrics'] = _OPMETRICS
DESCRIPTOR.message_types_by_name['PrecisionStats'] = _PRECISIONSTATS
DESCRIPTOR.message_types_by_name['OpMetricsDb'] = _OPMETRICSDB
DESCRIPTOR.enum_types_by_name['LayoutDimensionSemantics'] = _LAYOUTDIMENSIONSEMANTICS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)
LayoutAnalysis = _reflection.GeneratedProtocolMessageType('LayoutAnalysis', (_message.Message,), {'Dimension': _reflection.GeneratedProtocolMessageType('Dimension', (_message.Message,), {'DESCRIPTOR': _LAYOUTANALYSIS_DIMENSION, '__module__': 'tensorflow.core.profiler.protobuf.op_metrics_pb2'}), 'DESCRIPTOR': _LAYOUTANALYSIS, '__module__': 'tensorflow.core.profiler.protobuf.op_metrics_pb2'})
_sym_db.RegisterMessage(LayoutAnalysis)
_sym_db.RegisterMessage(LayoutAnalysis.Dimension)
OpMetrics = _reflection.GeneratedProtocolMessageType('OpMetrics', (_message.Message,), {'MemoryAccessed': _reflection.GeneratedProtocolMessageType('MemoryAccessed', (_message.Message,), {'DESCRIPTOR': _OPMETRICS_MEMORYACCESSED, '__module__': 'tensorflow.core.profiler.protobuf.op_metrics_pb2'}), 'DESCRIPTOR': _OPMETRICS, '__module__': 'tensorflow.core.profiler.protobuf.op_metrics_pb2'})
_sym_db.RegisterMessage(OpMetrics)
_sym_db.RegisterMessage(OpMetrics.MemoryAccessed)
PrecisionStats = _reflection.GeneratedProtocolMessageType('PrecisionStats', (_message.Message,), {'DESCRIPTOR': _PRECISIONSTATS, '__module__': 'tensorflow.core.profiler.protobuf.op_metrics_pb2'})
_sym_db.RegisterMessage(PrecisionStats)
OpMetricsDb = _reflection.GeneratedProtocolMessageType('OpMetricsDb', (_message.Message,), {'DESCRIPTOR': _OPMETRICSDB, '__module__': 'tensorflow.core.profiler.protobuf.op_metrics_pb2'})
_sym_db.RegisterMessage(OpMetricsDb)



from . import core
from . import file_system
from . import table_options
from . import block_builder
from . import errors
from . import table
from . import coding
from . import strings
from copy import copy

# void FindShortestSeparator(string* start, const StringPiece& limit) {
def find_shortest_separator(start: bytearray, limit: core.StringPiece):
  # // Find length of common prefix
  # size_t min_length = std::min(start->size(), limit.size());
  min_length = min(len(start), limit.size())
  # size_t diff_index = 0;
  diff_index = 0
  # while ((diff_index < min_length) &&
  #        ((*start)[diff_index] == limit[diff_index])) {
  while ((diff_index < min_length) and
         (start[diff_index] == limit[diff_index])):
    # diff_index++;
    diff_index += 1
  # }

  # if (diff_index >= min_length) {
  if diff_index >= min_length:
    # // Do not shorten if one string is a prefix of the other
    pass
  # } else {
  else:
    # uint8 diff_byte = static_cast<uint8>((*start)[diff_index]);
    diff_byte = start[diff_index]
    # if (diff_byte < static_cast<uint8>(0xff) &&
    #     diff_byte + 1 < static_cast<uint8>(limit[diff_index])) {
    if (diff_byte < 0xFF and
        diff_byte + 1 < limit[diff_index]):
      # (*start)[diff_index]++;
      start[diff_index] += 1

      # start->resize(diff_index + 1);
      strings.resize(start, diff_index + 1)
      # assert(StringPiece(*start).compare(limit) < 0);
      assert core.StringPiece(start).compare(limit) < 0
    # }
  # }
# }
#
# void FindShortSuccessor(string* key) {
def find_short_successor(key: bytearray):
  # // Find first character that can be incremented
  # size_t n = key->size();
  n = len(key)
  # for (size_t i = 0; i < n; i++) {
  for i in range(n):
    # const uint8 byte = (*key)[i];
    byte = key[i]
    # if (byte != static_cast<uint8>(0xff)) {
    if byte != 0xFF:
      # (*key)[i] = byte + 1;
      key[i] = byte + 1
      # key->resize(i + 1);
      strings.resize(key, i + 1)
      # return;
      return
    # }
  # }
  # // *key is a run of 0xffs.  Leave it alone.
# }

class Rep:
  def __init__(self, opt: table_options.Options, f: file_system.WritableFile):
    self.options = copy(opt)
    self.index_block_options = copy(opt)
    self.file = f
    self.offset = 0
    self.status = errors.Status.OK()
    self.data_block = block_builder.BlockBuilder(self.options)
    self.index_block = block_builder.BlockBuilder(self.index_block_options)
    self.last_key = bytearray()
    self.num_entries = 0
    self.closed = False  # Either Finish() or Abandon() has been called.

    # We do not emit the index entry for a block until we have seen the
    # first key for the next data block.  This allows us to use shorter
    # keys in the index block.  For example, consider a block boundary
    # between the keys "the quick brown fox" and "the who".  We can use
    # "the r" as the key for the index block entry since it is >= all
    # entries in the first block and < all entries in subsequent
    # blocks.
    #
    # Invariant: r->pending_index_entry is true only if data_block is empty.
    self.pending_index_entry = False
    self.pending_handle = table.BlockHandle()  # Handle to add to index block

    self.compressed_output = bytearray()

    self.index_block_options.block_restart_interval = 1

# class TableBuilder {
#  public:
class TableBuilder:
  """TableBuilder provides the interface used to build a Table
  (an immutable and sorted map from keys to values).

  Multiple threads can invoke const methods on a TableBuilder without
  external synchronization, but if any of the threads may call a
  non-const method, all threads accessing the same TableBuilder must use
  external synchronization.
  """
  # // Create a builder that will store the contents of the table it is
  # // building in *file.  Does not close the file.  It is up to the
  # // caller to close the file after calling Finish().
  # TableBuilder(const Options& options, WritableFile* file);
  def __init__(self, options: table_options.Options, file: file_system.WritableFile):
    """Create a builder that will store the contents of the table it is
    building in *file.  Does not close the file.  It is up to the
    caller to close the file after calling Finish()."""
    self.rep_ = Rep(options, file)

  # // REQUIRES: Either Finish() or Abandon() has been called.
  # ~TableBuilder();
  #
  # // Add key,value to the table being constructed.
  # // REQUIRES: key is after any previously added key in lexicographic order.
  # // REQUIRES: Finish(), Abandon() have not been called
  # void Add(const StringPiece& key, const StringPiece& value);
  def add(self, key: core.StringPiece, value: core.StringPiece):
    key = core.string_view(key)
    value = core.string_view(value)
    # Rep* r = rep_;
    r = self.rep_
    # assert(!r->closed);
    assert not r.closed
    # if (!ok()) return;
    errors.raise_if_error(self.status())
    # if (r->num_entries > 0) {
    if r.num_entries > 0:
      # assert(key.compare(StringPiece(r->last_key)) > 0);
      assert key.compare(core.StringPiece(r.last_key)) > 0
      # // See if this key+value would make our current block overly large.  If
      # // so, emit the current block before adding this key/value
      # const int kOverlyLargeBlockRatio = 2;
      kOverlyLargeBlockRatio = 2
      # const size_t this_entry_bytes = key.size() + value.size();
      this_entry_bytes = key.size() + value.size()
      # if (this_entry_bytes >= kOverlyLargeBlockRatio * r->options.block_size) {
      if this_entry_bytes >= kOverlyLargeBlockRatio * r.options.block_size:
        # Flush();
        self.flush()
      # }
    # }

    # if (r->pending_index_entry) {
    if r.pending_index_entry:
      # assert(r->data_block.empty());
      assert r.data_block.empty()
      # FindShortestSeparator(&r->last_key, key);
      find_shortest_separator(r.last_key, key)
      # string handle_encoding;
      # r->pending_handle.EncodeTo(&handle_encoding);
      handle_encoding = r.pending_handle.encode_to()
      # r->index_block.Add(r->last_key, StringPiece(handle_encoding));
      r.index_block.add(r.last_key, core.StringPiece(handle_encoding))
      # r->pending_index_entry = false;
      r.pending_index_entry = False
    # }
    #
    # r->last_key.assign(key.data(), key.size());
    r.last_key[:] = key.bytes()
    # r->num_entries++;
    r.num_entries += 1
    # r->data_block.Add(key, value);
    r.data_block.add(key, value)
    #
    # const size_t estimated_block_size = r->data_block.CurrentSizeEstimate();
    estimated_block_size = r.data_block.current_size_estimate()
    # if (estimated_block_size >= r->options.block_size) {
    if estimated_block_size >= r.options.block_size:
      # Flush();
      self.flush()
    # }


  # // Advanced operation: writes any buffered key/value pairs to file.
  # // Can be used to ensure that two adjacent entries never live in
  # // the same data block.  Most clients should not need to use this method.
  # // Does not flush the file itself.
  # // REQUIRES: Finish(), Abandon() have not been called
  # void Flush();
  def flush(self):
    """Advanced operation: writes any buffered key/value pairs to file.
    Can be used to ensure that two adjacent entries never live in
    the same data block.  Most clients should not need to use this method.
    Does not flush the file itself.

    REQUIRES: self.finish(), self.abandon() have not been called
    """
    # Rep* r = rep_;
    r = self.rep_
    # assert(!r->closed);
    assert not r.closed
    # if (!ok()) return;
    if not self.ok():
      return
    # if (r->data_block.empty()) return;
    if r.data_block.empty():
      return
    # assert(!r->pending_index_entry);
    assert not r.pending_index_entry
    # WriteBlock(&r->data_block, &r->pending_handle);
    self.write_block(r.data_block, r.pending_handle)
    # if (ok()) {
    if self.ok():
      # r->pending_index_entry = true;
      r.pending_index_entry = True
      # // We don't flush the underlying file as that can be slow.
    # }


  # // Return non-ok iff some error has been detected.
  # Status status() const;
  def status(self):
    """Return non-ok iff some error has been detected."""
    return self.rep_.status

  # // Finish building the table.  Stops using the file passed to the
  # // constructor after this function returns.
  # // REQUIRES: Finish(), Abandon() have not been called
  # Status Finish();
  def finish(self) -> errors.Status:
    """Finish building the table.  Stops using the file passed to the
    constructor after this function returns.

    REQUIRES: Finish(), Abandon() have not been called
    """
    # Rep* r = rep_;
    r = self.rep_
    # Flush();
    self.flush()
    # assert(!r->closed);
    assert not r.closed
    # r->closed = true;
    r.closed = True

    # BlockHandle metaindex_block_handle, index_block_handle;
    metaindex_block_handle = table.BlockHandle()
    index_block_handle = table.BlockHandle()

    # // Write metaindex block
    # if (ok()) {
    if self.ok():
      # BlockBuilder meta_index_block(&r->options);
      meta_index_block = block_builder.BlockBuilder(r.options)
      # // TODO(postrelease): Add stats and other meta blocks
      # WriteBlock(&meta_index_block, &metaindex_block_handle);
      self.write_block(meta_index_block, metaindex_block_handle)
    # }

    # // Write index block
    # if (ok()) {
    if self.ok():
      # if (r->pending_index_entry) {
      if r.pending_index_entry:
        # FindShortSuccessor(&r->last_key);
        find_short_successor(r.last_key)
        # string handle_encoding;
        # r->pending_handle.EncodeTo(&handle_encoding);
        handle_encoding = r.pending_handle.encode_to()
        # r->index_block.Add(r->last_key, StringPiece(handle_encoding));
        r.index_block.add(r.last_key, core.StringPiece(handle_encoding))
        # r->pending_index_entry = false;
        r.pending_index_entry = False
      # }
      # WriteBlock(&r->index_block, &index_block_handle);
      self.write_block(r.index_block, index_block_handle)
    # }

    # // Write footer
    # if (ok()) {
    if self.ok():
      # Footer footer;
      footer = table.Footer()
      # footer.set_metaindex_handle(metaindex_block_handle);
      footer.set_metaindex_handle(metaindex_block_handle)
      # footer.set_index_handle(index_block_handle);
      footer.set_index_handle(index_block_handle)
      # string footer_encoding;
      # footer.EncodeTo(&footer_encoding);
      footer_encoding = footer.encode_to()
      # r->status = r->file->Append(footer_encoding);
      r.status = r.file.append(footer_encoding)
      # if (r->status.ok()) {
      if r.status.ok():
        # r->offset += footer_encoding.size();
        r.offset += len(footer_encoding)
      # }
    # }
    # return r->status;
    return r.status

  #
  # // Indicate that the contents of this builder should be abandoned.  Stops
  # // using the file passed to the constructor after this function returns.
  # // If the caller is not going to call Finish(), it must call Abandon()
  # // before destroying this builder.
  # // REQUIRES: Finish(), Abandon() have not been called
  # void Abandon();
  #
  # // Number of calls to Add() so far.
  # uint64 NumEntries() const;
  #
  # // Size of the file generated so far.  If invoked after a successful
  # // Finish() call, returns the size of the final generated file.
  # uint64 FileSize() const;

  #private:
  # bool ok() const { return status().ok(); }
  def ok(self):
    return self.status().ok()

  # void WriteBlock(BlockBuilder* block, BlockHandle* handle);
  def write_block(self, block: block_builder.BlockBuilder, handle: table.BlockHandle):
    # // File format contains a sequence of blocks where each block has:
    # //    block_data: uint8[n]
    # //    type: uint8
    # //    crc: uint32
    # assert(ok());
    assert self.ok()
    # Rep* r = rep_;
    r = self.rep_
    # StringPiece raw = block->Finish();
    raw = block.finish()

    # StringPiece block_contents;
    # CompressionType type = r->options.compression;
    type = r.options.compression
    # // TODO(postrelease): Support more compression options: zlib?
    # switch (type) {
    #   case kNoCompression:
    if type == table_options.kNoCompression:
      # block_contents = raw;
      block_contents = raw
      # break;
    #
    #   case kSnappyCompression: {
    elif type == table_options.kSnappyCompression:
      # string* compressed = &r->compressed_output;
      # if (port::Snappy_Compress(raw.data(), raw.size(), compressed) &&
      #     compressed->size() < raw.size() - (raw.size() / 8u)) {
      #   block_contents = *compressed;
      # } else {
      #   // Snappy not supported, or compressed less than 12.5%, so just
      #   // store uncompressed form
      #   block_contents = raw;
      #   type = kNoCompression;
      # }
      # break;
      raise NotImplementedError("TODO: Support SnappyCompression")
    #   }
    # }
    # WriteRawBlock(block_contents, type, handle);
    self.write_raw_block(block_contents, type, handle)
    # r->compressed_output.clear();
    r.compressed_output.clear()
    # block->Reset();
    block.reset()

  # void WriteRawBlock(const StringPiece& data, CompressionType,
  #                    BlockHandle* handle);
  def write_raw_block(self, block_contents: core.StringPiece, type: table_options.CompressionType, handle: table.BlockHandle):
    # Rep* r = rep_;
    r = self.rep_
    # handle->set_offset(r->offset);
    handle.set_offset(r.offset)
    # handle->set_size(block_contents.size());
    handle.set_size(block_contents.size())
    # r->status = r->file->Append(block_contents);
    r.status = r.file.append(block_contents)
    # if (r->status.ok()) {
    if r.status.ok():
      # char trailer[kBlockTrailerSize];
      trailer = bytearray(table.kBlockTrailerSize)
      # trailer[0] = type;
      trailer[0] = type
      # uint32 crc = crc32c::Value(block_contents.data(), block_contents.size());
      crc = core.crc32c.value(block_contents.data(), block_contents.size())
      # crc = crc32c::Extend(crc, trailer, 1);  // Extend crc to cover block type
      crc = core.crc32c.extend(crc, trailer, 1)  # Extend crc to cover block type
      # core::EncodeFixed32(trailer + 1, crc32c::Mask(crc));
      trailer[1:5] = coding.encode_fixed32(core.crc32c.mask(crc))
      # r->status = r->file->Append(StringPiece(trailer, kBlockTrailerSize));
      r.status = r.file.append(core.StringPiece(trailer, table.kBlockTrailerSize))
      # if (r->status.ok()) {
      if r.status.ok():
        # r->offset += block_contents.size() + kBlockTrailerSize;
        r.offset += block_contents.size() + table.kBlockTrailerSize
      # }
    # }

  #
  # struct Rep;
  # Rep* rep_;
  #
  # // No copying allowed
  # TableBuilder(const TableBuilder&);
  # void operator=(const TableBuilder&);
# };
